﻿	
	/****************************************************
	 * 	Script:			connection.js					*
	 *	Description:	Function to connect to server	*
	 *	Author:			Max Power						*
	 *	Version:		1.0								*
	 *	Date:			01/03/2010						*
	 ****************************************************/
	
	const RESULT_TYPE_BYTES   = 0;
	const RESULT_TYPE_TEXT    = 1;
	const RESULT_TYPE_XML     = 2;
	const REQUEST_COMPLETE    = 4;
	const STATUS_RESQUEST_OK  = 200;
	
	const EMPTY_STRING        = '';
	const PARAMETER_SEPARATOR = '&';
	const PARAMETER_EQUALS    = '=';
	const METHOD_TYPE_POST    = 'POST';
	const MSXML2_XMLHTTP      = 'MSXML2.XMLHTTP';
	const MICROSOFT_XMLHTTP   = 'Microsoft.XMLHTTP';
	const ATTR_CONTENT_TYPE   = 'Content-type';
	const ATTR_CONTENT_LENGTH = 'Content-length';
	const VALUE_TEXT_XML      = 'text/xml';
	const VALUE_CONTENT_TYPE  = 'application/x-www-form-urlencoded';
	
	/**
	 *	Class:			Connection
	 *	Description:	Creates a connection and send a request to the server
	 **/
	
	function Connection() {
	}
	
	/**
	 *	Create a connector to send a request
	 **/
	
	Connection.createConnector = function() {
		var request = null;
		
		if (window.XMLHttpRequest)
			request = new XMLHttpRequest();
		else if (window.ActiveXObject) {
			try {
				request = new ActiveXObject(MSXML2_XMLHTTP);
			} catch (event) {
				try {
					request = new ActiveXObject(MICROSOFT_XMLHTTP);
				} catch (event) {
				}
			}
		}
		
		return request;
	}
	
	/**
	 *	Send a request to server and wait for response
	 **/
	
	Connection.send = function(url, complete, fail, parameters, resultType) {
		var request = Connection.createConnector();
		
		if (request != null) {
			
			request.open(METHOD_TYPE_POST, url, true);
			request.onabort = fail;
			
			var arguments = EMPTY_STRING;
			
			if (parameters != null) {
				arguments = parameters.get();
				request.setRequestHeader(ATTR_CONTENT_TYPE,   VALUE_CONTENT_TYPE);
				request.setRequestHeader(ATTR_CONTENT_LENGTH, arguments.length);
			} else
				arguments = null;
			
			request.onreadystatechange = function () {
				
				if (request.readyState == REQUEST_COMPLETE) {
					if (request.status == STATUS_RESQUEST_OK) {
						switch (resultType) {
							case RESULT_TYPE_BYTES:
								complete(request.responseBody);
								break;
							case RESULT_TYPE_TEXT:
								complete(request.responseText);
								break;
							case RESULT_TYPE_XML:
								var xml = (new DOMParser()).parseFromString(request.responseText, VALUE_TEXT_XML);
								complete(xml.documentElement);
								break;
						}
					} else
						fail();
				}
			}
			
			request.send(arguments);
		} else
			fail();
	}
	
	/**
	 *	Class:			Parameters
	 *	Description:	Creates the parameters to be send to the server
	 **/
	
	function Parameters() {
		
		this.data = new Table();
		
		/**
		 *	Add a key and a value to parameters
		 **/
		
		this.add = function(key, value) {
			this.data.put(key, value);
		}
		
		/**
		 *	Returns the string representing the parameters
		 **/
		
		this.get = function() {
			var keys   = this.data.keys();
			var params = EMPTY_STRING;
			
			for (var i = 0; i < keys.length; i++) {
				var key = keys[i];
				
				if (params != EMPTY_STRING)
					params += PARAMETER_SEPARATOR;
				
				params += key + PARAMETER_EQUALS + escape(this.data.get(key));
			}
			
			return params;
		}
	}