﻿	
	/****************************************************
	 * 	Script:			node.js							*
	 *	Description:	Node for the tree				*
	 *	Author:			Max Power						*
	 *	Version:		1.0								*
	 *	Date:			01/03/2010						*
	 ****************************************************/
	
	/**
	 *	Class:			Node
	 *	Description:	Node for the tree
	 **/
	
	function Node(name, path, isFolder, father) {
		
		this.name     = name;
		this.path     = path;
		this.isFolder = isFolder;
		this.father   = father;
		this.opened   = false;
		this.row      = null;
		
		if (this.isFolder)
			this.childrens = new Array();
		else
			this.childrens = null;
		
		if (this.father != null)
			this.father.addChildren(this);
		
		/**
		 *	Return true if the node is root
		**/
		
		this.isRoot = function() {
			return (this.father == null);
		}
		
		/**
		 *	Set the row where the node is contained
		**/
		
		this.setRow = function(newRow) {
			this.row = newRow;
		}
		
		/**
		 *	Returns the row where the node is contained
		**/
		
		this.getRow = function() {
			return this.row;
		}
		
		/**
		 *	Delete the children
		**/
		
		this.deleteChildren = function(children) {
			this.childrens.splice(this.childrens.indexOf(children), 1);
		}
		
		/**
		 *	Delete the node
		**/
		
		this.deleteNode = function() {
			this.father.deleteChildren(this);
		}
		
		/**
		 *	Returns the internal path of the node in the project
		**/
		
		this.getInternalPath = function() {
			var path = EMPTY_STRING;
			
			if (this.father != null)
				path += father.getInternalPath();
			
			path += PATH_SEPARATOR + this.name;
			
			return path;
		}
		
		/**
		 *	Returns the name of the node
		**/
		
		this.getName = function() {
			return (this.name);
		}
		
		/**
		 *	Returns the path of the node into server
		**/
		
		this.getPath = function() {
			return (this.path);
		}
		
		/**
		 *	Return true if the node is a folder
		**/
		
		this.isDirectory = function() {
			return (this.isFolder);
		}
		
		/**
		 *	Return true if the node has childrens
		**/
		
		this.hasChildren = function() {
			return (this.childrens != null);
		}
		
		/**
		 *	Return true if the node is opened
		**/
		
		this.isOpen = function() {
			return this.opened;
		}
		
		/**
		 *	Open or close the node
		**/
		
		this.action = function() {
			if (this.opened)
				this.close();
			else
				this.open();
		}
		
		/**
		 *	Open the node
		**/
		
		this.open = function() {
			this.opened = true;
		}
		
		/**
		 *	Close the node
		**/
		
		this.close = function() {
			this.opened = false;
		}
		
		/**
		 *	Returns the children of the node
		**/
		
		this.getChildren = function() {
			return (this.childrens);
		}
		
		/**
		 *	Returns the father of the node
		**/
		
		this.getFather = function() {
			return (this.father);
		}
		
		/**
		 *	Add a new children to the node
		**/
		
		this.addChildren = function(children) {
			return (this.childrens.push(children));
		}
	}
	