﻿<?php
	
	/****************************************************
	 * 	Script:			script.php						*
	 *	Description:	Server program					*
	 *	Author:			Max Power						*
	 *	Version:		1.0								*
	 *	Date:			01/03/2010						*
	 ****************************************************/
	
	/*************** OPERATION TYPES ***************/
	define('OPERATION_LOAD',     'load');
	define('OPERATION_SAVE',     'save');
	define('OPERATION_LOGIN',    'login');
	define('OPERATION_PROJECTS', 'projects');
	define('OPERATION_DELETE',   'delete');
	define('OPERATION_RENAME',   'renameElement');
	define('OPERATION_REFRESH',  'refresh');
	define('OPERATION_NEW_DIR',  'newDir');
	define('OPERATION_NEW_FILE', 'newFile');
	
	/*************** PARAMETERS TYPES ***************/
	define('PARAM_OPERATION',     'operation');
	define('PARAM_FILE',          'file');
	define('PARAM_CONTENT',       'content');
	define('PARAM_USER',          'user');
	define('PARAM_PASSWORD',      'password');
	define('PARAM_REFERENCE',     'reference');
	define('PARAM_OLD_REFERENCE', 'oldReference');
	define('PARAM_NEW_REFERENCE', 'newReference');
	define('PARAM_PROJECT',       'project');
	define('PARAM_PROJECT_NAME',  'projectName');
	
	/****************** XML TYPES ******************/
	define('XML_TAG_HEADER', "<?xml version='1.0' encoding='UTF-8'?>");
	
	//******************** LOGIN TAGS ********************/
	define('XML_TAG_LOGIN_OPEN',  '<login>');
	define('XML_TAG_LOGIN_CLOSE', '</login>');
	define('XML_TAG_LOGIN_OK',    '<loginOk/>');
	define('XML_TAG_LOGIN_FAIL',  '<loginFail/>');
	
	//***************** CREATE DIR TAGS ******************/
	define('XML_TAG_CREATE_DIR_OPEN',  '<createDir>');
	define('XML_TAG_CREATE_DIR_CLOSE', '</createDir>');
	define('XML_TAG_CREATE_DIR_OK',    '<createDirOk/>');
	define('XML_TAG_CREATE_DIR_FAIL',  '<createDirFail/>');
	
	//***************** CREATE FILE TAGS *****************/
	define('XML_TAG_CREATE_FILE_OPEN',  '<createFile>');
	define('XML_TAG_CREATE_FILE_CLOSE', '</createFile>');
	define('XML_TAG_CREATE_FILE_OK',    '<createFileOk/>');
	define('XML_TAG_CREATE_FILE_FAIL',  '<createFileFail/>');
	
	//******************* RENAME TAGS ********************/
	define('XML_TAG_RENAME_OPEN',  '<rename>');
	define('XML_TAG_RENAME_CLOSE', '</rename>');
	define('XML_TAG_RENAME_OK',    '<renameOk/>');
	define('XML_TAG_RENAME_FAIL',  '<renameFail/>');
	
	//******************* DELETE TAGS ********************/
	define('XML_TAG_DELETE_OPEN',  '<delete>');
	define('XML_TAG_DELETE_CLOSE', '</delete>');
	define('XML_TAG_DELETE_OK',    '<deleteOk/>');
	define('XML_TAG_DELETE_FAIL',  '<deleteFail/>');
	
	//******************** SAVE TAGS *********************/
	define('XML_TAG_SAVE_OPEN',  '<save>');
	define('XML_TAG_SAVE_CLOSE', '</save>');
	define('XML_TAG_SAVE_OK',    '<saveOk/>');
	define('XML_TAG_SAVE_FAIL',  '<saveFail/>');
	
	//******************* PROJECTS TAGS ******************/
	define('XML_TAG_PROJECTS_OPEN',  '<projects>');
	define('XML_TAG_PROJECTS_CLOSE', '</projects>');
	define('XML_TAG_PROJECT_CLOSE',  '</project>');
	define('XML_TAG_FOLDER_CLOSE',   '</folder>');
	define('XML_TAG_PROJECT_LEFT',   "<project name='");
	define('XML_TAG_FOLDER_LEFT',    "<folder name='");
	define('XML_TAG_FILE_LEFT',      "<file name='");
	define('XML_TAG_MIDDLE',         "' path='");
	define('XML_TAG_FILE_RIGHT',     "'/>");
	define('XML_TAG_RIGHT',          "'>");
	
	/****************** FILE TYPES ******************/
	define('EMPTY_STRING',       '');
	define('FILE_OPEN_MODE',     'w');
	define('CURRENT_DIR',        '.');
	define('PARENT_DIR',         '..');
	define('PATH_DIR_SEPARATOR', '/');
	
	$operation = getParameter(PARAM_OPERATION);
	
	switch ($operation) {
		
		case OPERATION_LOAD:
			load();
			break;
			
		case OPERATION_SAVE:
			save();
			break;
			
		case OPERATION_LOGIN:
			login();
			break;
			
		case OPERATION_PROJECTS:
			projects();
			break;
			
		case OPERATION_DELETE:
			delete();
			break;
			
		case OPERATION_RENAME:
			renameElement();
			break;
			
		case OPERATION_REFRESH:
			refresh();
			break;
			
		case OPERATION_NEW_DIR:
			newDir();
			break;
			
		case OPERATION_NEW_FILE:
			newFile();
			break;
	}
	
	/**
	 *	Return the value of the parameter
	**/
	
	function getParameter($name) {
		return urldecode($_REQUEST[$name]);
	}
	
	/**
	 *	Load and return the content of the file
	**/
	
	function load() {
		$user     = getParameter(PARAM_USER);
		$password = getParameter(PARAM_PASSWORD);
		$file     = getParameter(PARAM_FILE);
		
		if (validUser($user, $password) && hasPermission($user, $file))
			echo file_get_contents($file);
	}
	
	/**
	 *	Save the content of the file
	**/
	
	function save() {
		$result   = XML_TAG_HEADER;
		$result  .= XML_TAG_SAVE_OPEN;
		
		$user     = getParameter(PARAM_USER);
		$password = getParameter(PARAM_PASSWORD);
		$file     = getParameter(PARAM_FILE);
		$content  = getParameter(PARAM_CONTENT);
		
		if (validUser($user, $password) && hasPermission($user, $file) && file_put_contents($file, $content))
			$result .= XML_TAG_SAVE_OK;
		else
			$result .= XML_TAG_SAVE_FAIL;
		
		$result .= XML_TAG_SAVE_CLOSE;
		
		echo $result;
	}
	
	/**
	 *	Validate the user
	**/
	
	function login() {
		$result   = XML_TAG_HEADER;
		$result  .= XML_TAG_LOGIN_OPEN;
		
		$user     = getParameter(PARAM_USER);
		$password = getParameter(PARAM_PASSWORD);
		
		if (validUser($user, $password))
			$result .= XML_TAG_LOGIN_OK;
		else
			$result .= XML_TAG_LOGIN_FAIL;
		
		$result .= XML_TAG_LOGIN_CLOSE;
		
		echo $result;
	}
	
	/**
	 *	Create a new folder
	**/
	
	function newDir() {
		$result    = XML_TAG_HEADER;
		$result   .= XML_TAG_CREATE_DIR_OPEN;
		
		$user      = getParameter(PARAM_USER);
		$password  = getParameter(PARAM_PASSWORD);
		$reference = getParameter(PARAM_REFERENCE);
		
		if ((validUser($user, $password)) && hasPermission($user, $reference) && createDir($reference))
			$result .= XML_TAG_CREATE_DIR_OK;
		else
			$result .= XML_TAG_CREATE_DIR_FAIL;
		
		$result .= XML_TAG_CREATE_DIR_CLOSE;
		
		echo $result;
	}
	
	/**
	 *	Create a new file
	**/
	
	function newFile() {
		$result    = XML_TAG_HEADER;
		$result   .= XML_TAG_CREATE_FILE_OPEN;
		
		$user      = getParameter(PARAM_USER);
		$password  = getParameter(PARAM_PASSWORD);
		$reference = getParameter(PARAM_REFERENCE);
		$content   = getParameter(PARAM_CONTENT);
		
		if ((validUser($user, $password)) && hasPermission($user, $reference) && createFile($reference, $content))
			$result .= XML_TAG_CREATE_FILE_OK;
		else
			$result .= XML_TAG_CREATE_FILE_FAIL;
		
		$result .= XML_TAG_CREATE_FILE_CLOSE;
		
		echo $result;
	}
	
	/**
	 *	Create a new folder
	**/
	
	function createDir($reference) {
		return mkdir($reference);
	}
	
	/**
	 *	Create a new file
	**/
	
	function createFile($reference, $content) {
		$valid = false;
	
		$handle = fopen($reference, FILE_OPEN_MODE);
		
		if ($handle) {
			fwrite($handle, $content);
			$valid = true;
		}
		
		fclose($handle);
		
		return $valid;
	}
	
	/**
	 *	Refresh the project
	**/
	
	function refresh() {
		$result      = XML_TAG_HEADER;
		$result     .= XML_TAG_PROJECTS_OPEN;
		
		$user        = getParameter(PARAM_USER);
		$password    = getParameter(PARAM_PASSWORD);
		$project     = getParameter(PARAM_PROJECT);
		$projectName = getParameter(PARAM_PROJECT_NAME);
		
		if (validUser($user, $password) && hasPermission($user, $project))
			$result .= readProject($projectName, $project);
		
		$result .= XML_TAG_PROJECTS_CLOSE;
		
		echo $result;
	}
	
	/**
	 *	Rename an element
	**/
	
	function renameElement() {
		$result       = XML_TAG_HEADER;
		$result      .= XML_TAG_RENAME_OPEN;
		
		$user         = getParameter(PARAM_USER);
		$password     = getParameter(PARAM_PASSWORD);
		$oldReference = getParameter(PARAM_OLD_REFERENCE);
		$newReference = getParameter(PARAM_NEW_REFERENCE);		
		
		if ((validUser($user, $password)) && hasPermission($user, $oldReference) && renameReference($oldReference, $newReference))
			$result .= XML_TAG_RENAME_OK;
		else
			$result .= XML_TAG_RENAME_FAIL;
		
		$result .= XML_TAG_RENAME_CLOSE;
		
		echo $result;
	}
	
	/**
	 *	Rename a reference
	**/
	
	function renameReference($oldReference, $newReference) {
		return rename($oldReference, dirname($oldReference) . PATH_DIR_SEPARATOR . $newReference);
	}
	
	/**
	 *	Delete a reference
	**/
	
	function delete() {
		$result    = XML_TAG_HEADER;
		$result   .= XML_TAG_DELETE_OPEN;
		
		$user      = getParameter(PARAM_USER);
		$password  = getParameter(PARAM_PASSWORD);
		$reference = getParameter(PARAM_REFERENCE);
		
		if ((validUser($user, $password)) && hasPermission($user, $reference) && deleteReference($reference))
			$result .= XML_TAG_DELETE_OK;
		else
			$result .= XML_TAG_DELETE_FAIL;
		
		$result .= XML_TAG_DELETE_CLOSE;
		
		echo $result;
	}
	
	/**
	 *	Delete a reference
	**/
	
	function deleteReference($reference) {
		if (is_dir($reference))
			return deleteDirectory($reference);
		else
			return deleteFile($reference);
	}
	
	/**
	 *	Delete a file
	**/
	
	function deleteFile($reference) {
		return unlink($reference);
	}
	
	/**
	 *	Delete a folder
	**/
	
	function deleteDirectory($reference) {
		$valid  = true;
		$handle = opendir($reference);
		  
		if (!$handle)
			return false;
		
		while ($file = readdir($handle)) {
			if (($file != CURRENT_DIR) && ($file != PARENT_DIR)) {
				
				$path = $reference . PATH_DIR_SEPARATOR . $file;
				
				if (is_dir($path))
					$valid = ($valid && deleteDirectory($path));
				else
					$valid = ($valid && deleteFile($path));
			}
		}
		
		closedir($handle);
		
		return ($valid && rmdir($reference));
	}
	
	/**
	 *	Validate the user
	**/
	
	function validUser($user, $password) {
		//TODO:
		return (($user == 'demo') && ($password == 'demo'));
	}
	
	/**
	 *	Return the projects of the user
	**/
	
	function userProjectsPermission($user) {
		//TODO:
		$result = array();
		
		if ($user == 'demo')
			$result['Demo Project'] = '/homepages/43/d322805772/htdocs/proyectos/maxeditor/demo project';
		
		return $result;
	}
	
	/**
	 *	Return the projects of the user
	**/
	
	function userProjects($user) {
		$projectsUser = userProjectsPermission($user);
		
		foreach ($projectsUser as $name => $path)
			$result .= readProject($name, $path);
			
		return $result;
	}
	
	/**
	 *	Return the projects of the user
	**/
	
	function hasPermission($user, $file) {
		$valid = false;
		$projectsUser = userProjectsPermission($user);
		
		foreach ($projectsUser as $path) {
			if (strripos($file, $path) === 0) {
				$valid = true;
				break;
			}
		}
		
		return $valid;
	}
	
	/**
	 *	Return the projects of the user
	**/
	
	function projects() {
		$result   = XML_TAG_HEADER;
		$result  .= XML_TAG_PROJECTS_OPEN;
		
		$user     = getParameter(PARAM_USER);
		$password = getParameter(PARAM_PASSWORD);
		
		if (validUser($user, $password))
			$result .= userProjects($user);
		
		$result .= XML_TAG_PROJECTS_CLOSE;
		
		echo $result;
	}
	
	/**
	 *	Return a project
	**/
	
	function readProject($name, $path) {
		return readFolder($path, $name, true);
	}
	
	/**
	 *	Return a project head tag
	**/
	
	function getProjectHeadTag($projectName, $dir) {		
		return XML_TAG_PROJECT_LEFT . $projectName . XML_TAG_MIDDLE . $dir . XML_TAG_RIGHT;
	}
	
	/**
	 *	Return a folder head tag
	**/
	
	function getFolderHeadTag($name, $dir) {
		return XML_TAG_FOLDER_LEFT . $name . XML_TAG_MIDDLE . $dir . XML_TAG_RIGHT;
	}
	
	/**
	 *	Return a file head tag
	**/
	
	function getFileHeadTag($fileName, $filePath) {
		return XML_TAG_FILE_LEFT . $fileName . XML_TAG_MIDDLE . $filePath . XML_TAG_FILE_RIGHT;
	}
	
	/**
	 *	Read a folder
	**/
	
	function readFolder($dir, $projectName, $root) {
		$result = EMPTY_STRING;
		
		if ($root)
			$result .= getProjectHeadTag($projectName, $dir);
		
		if ($dirHandler = opendir($dir)) {
			
			$name = basename($dir);
			
			if (!$root)
				$result .= getFolderHeadTag($name, $dir);
			
			$files   = array();
			$folders = array();
		
			while (($file = readdir($dirHandler)) !== false) {
				
				if (($file != CURRENT_DIR) && ($file != PARENT_DIR)) {
					
					$path = $dir . PATH_DIR_SEPARATOR . $file;
					
					if (is_dir($path))
						$folders[$path] = $path;
					else
						$files[$file] = $path;
				}
			}
			
			ksort($folders);
			ksort($files);
			
			foreach ($folders as $folder)
				$result .= readFolder($folder, EMPTY_STRING, false);
			
			foreach ($files as $fileName => $filePath)
				$result .= getFileHeadTag($fileName, $filePath);
			
			closedir($dirHandler);
			
			if (!$root)
				$result .= XML_TAG_FOLDER_CLOSE;
		}
		
		if ($root)
			$result .= XML_TAG_PROJECT_CLOSE;
		
		return $result;
	}
?>